        .include "iom169p.inc"
		.include "macros.inc"
		.include "struct_gas.inc"

;       PB7 PB6 PB5
;PG0    SW1 SW2 SW3
;PG1     -  SW4 SW5

;R12-15     LCD content
;R0         0 constant
;R1         key scan
;R3         0x03 constant
;R8         0x1C constant
;R4         t
;R5         t-1
;R6         t-2
;R9         t
;R10        t-1
;R11        t-2

        .data
;dummy:              .space 0xC0
display_buffer:     .space 4
;delay:              .space 2

        .text
		;.org 0

		jmp ON_RESET
		jmp EXT_INT0_ISR     ; External Interrupt0
		jmp PCINT0_ISR       ; Pin Change Interrupt0
		jmp PCINT1_ISR       ; Pin Change Interrupt1
		jmp TIM2_COMP_ISR    ; Timer2 Compare
		jmp TIM2_OVF_ISR     ; Timer2 Overflow
		jmp TIM1_CAPT_ISR    ; Timer1 Capture
		jmp TIM1_COMPA_ISR   ; Timer1 CompareA
		jmp TIM1_COMPB_ISR   ; Timer1 CompareB
		jmp TIM1_OVF_ISR     ; Timer1 Overflow
		jmp TIM0_COMP_ISR    ; Timer0 Compare
		jmp TIM0_OVF_ISR     ; Timer0 Overflow
		jmp SPI_STC_ISR      ; SPI Transfer Complete
		jmp USART_RXC_ISR    ; USART, Rx Complete
		jmp USART_UDRE_ISR   ; USART, UDR Empty
		jmp USART_TXC_ISR    ; USART, Tx Complete
		jmp USI_START_ISR    ; USI Start Condition
		jmp USI_OVF_ISR      ; USI Overflow
		jmp ANA_COMP_ISR     ; Analog Comparator
		jmp ADC_ISR          ; ADC Conversion Complete
		jmp EE_RDY_ISR       ; EEPROM Ready
		jmp SPM_RDY_ISR      ; Store Program Memory Ready
		jmp LCD_SOF_ISR     ; LCD Start of Frame

EXT_INT0_ISR:     ; External Interrupt0 Handler
PCINT0_ISR:       ; Pin Change Interrupt0 Handler
PCINT1_ISR:       ; Pin Change Interrupt1 Handler
TIM2_COMP_ISR:    ; Timer2 Compare Handler
TIM2_OVF_ISR:     ; Timer2 Overflow Handler
TIM1_CAPT_ISR:    ; Timer1 Capture Handler
TIM1_COMPA_ISR:   ; Timer1 CompareA Handler
TIM1_COMPB_ISR:   ; Timer1 CompareB Handler
TIM1_OVF_ISR:     ; Timer1 Overflow Handler
TIM0_OVF_ISR:     ; Timer0 Overflow Handler
SPI_STC_ISR:      ; SPI Transfer Complete Handler
USART_RXC_ISR:    ; USART, Rx Complete Handler
USART_UDRE_ISR:   ; USART, UDR Empty Handler
USART_TXC_ISR:    ; USART, Tx Complete Handler
USI_START_ISR:    ; USI Start Condition Handler
USI_OVF_ISR:      ; USI Overflow Handler
ANA_COMP_ISR:     ; Analog Comparator Handler
ADC_ISR:          ; ADC Conversion Complete Handler
EE_RDY_ISR:       ; EEPROM Ready Handler
SPM_RDY_ISR:      ; Store Program Memory Ready Handler
        reti

TIM0_COMP_ISR:    ; Timer0 Compare Handler executed each 500ms
        ISR_PUSH_SREG_R16

;       PB7 PB6 PB5
;PG0    SW1 SW2 SW3
;PG1     -  SW4 SW5

;Scan lines by pulling low either PG0 or PG1 and reading the result on PB7..5
;Use DDR instead of PORTG to prevent shorting between PG0 and PG1 (only one used as output)
;PG0
;DDRG   xxxxxx01      PORTG     xxxxxx10
;PG1
;DDRG   xxxxxx10      PORTG     xxxxxx01

;R4     - - - SW1 SW2 SW3 SW4 SW5
        mov     r6,r5           ;t -> t-1 -> t-2
        mov     r5,r4

        in      r16,PINB
        com     r16

        sbrc    r1,1
        rjmp    TIM0_COMP_ISR_1
        ;_if
        ;PG0 = 0
            asr     r16         ;654
            asr     r16         ;543
            asr     r16         ;432
            and     r16,r8

            and     r4,r3       ;keep SW4 SW5
            or      r4,r16      ;combine SW1 SW2 SW3

            rjmp    TIM0_COMP_ISR_2

TIM0_COMP_ISR_1:
        ;_else
        ;PG1 = 0
            swap    r16         ;32107654
            asr     r16         ;.3210765 cy
            and     r16,r3      ;65

            and     r4,r8       ;keep SW1 SW2 SW3
            or      r4,r16      ;combine SW4 SW5
        ;_endif
TIM0_COMP_ISR_2:
        out     PORTG,r1
        eor     r1,r3
        out     DDRG,r1

		ISR_POP_SREG_R16_RETI

LCD_SOF_ISR:      ; LCD Start of Frame Handler
        ISR_PUSH_SREG_R16

        sts     LCDDR0,r12
        sts     LCDDR5,r13
        sts     LCDDR10,r14
        sts     LCDDR15,r15

		ISR_POP_SREG_R16_RETI

ON_RESET:
		OUT_PORT16	SP,RAMEND

        clr     r0              ;constant 0
        LD_IMM  r3,0x03         ;constant 3
        LD_IMM  r8,0x1C         ;constant 1C
        LD_IMM  r1,0x01

        OUT_PORT CLKPR,0x80
        OUT_PORT CLKPR,0x00     ;CPU CLK 8MHz

        ;initialize port for reading switches
        OUT_PORT DDRB,0x10      ;On board LED
        OUT_PORT PORTB,0xF0     ;7-5: pull-up on switch inputs, 4: LED output

        out DDRG,r1             ;switch scan lines
        inc     r1
        out PORTG,r1
        dec     r1

        ;disable clock on unused modules
        OUT_PORT PRR,0x0F       ;4-0: LCD, TIM1, SPI, USART, ADC

        ;timer 0 interrupts each 0.5ms
        OUT_PORT OCR0A,65
        OUT_PORT TCCR0A,0x0B    ;CTC, no OCR0A output, Osc/64
        OUT_PORT TIMSK0,0x02    ;overflow int

        ;for user app
        OUT_PORT DDRD,0xFF
        OUT_PORT PORTD,0x00

        rcall   init_lcd
        sei

        ;user application

1:      rcall   kbrd
        out     PORTD,r16
        ;tst     r4
        ;breq    2f
        ;out     PORTD,r4
2:
        ;OUT_PORT PORTD,0x02

        rjmp    1b




        CLR_T
loop:   rcall   delay1s
        rcall   dis_Th
        INC_T
		rjmp	loop

delay1s:
        PUSH_T
        LDI_T   1000
        rcall   delay
        POP_T
        ret

delay:  rcall   delay1ms
        dec     r18
        brne    delay
        dec     r19
        brne    delay
        ret

delay1ms:
        PUSH_T
        LDI_T   2950
1:      dec     r18
        brne    1b
        dec     r19
        brne    1b
        POP_T
        ret

init_lcd:
        ;169P
        ;OUT_PORT LCDDR15,0xFC
        ;OUT_PORT LCDDR10,0xF6
        ;OUT_PORT LCDDR5,0x6E
        ;OUT_PORT LCDDR0,0x14

        ldi r16,0xFC
        mov r15,r16
        ldi r16,0xF6
        mov r14,r16
        ldi r16,0x6E
        mov r13,r16
        ldi r16,0x14
        mov r12,r16

        ;Lumex S401M16KR (4 7-segment digit)
        ;1/4 duty, 1/3 bias
        ;LCDMUX1..0 = 11
        OUT_PORT LCDCRB,0x30

        OUT_PORT LCDFRR,0x63
        ;0x73
        OUT_PORT LCDCCR,0xE8

        OUT_PORT LCDCRA,0x88    ;7-0: LCDEN,LCDAB,-,LCDIF,LCDIE,LCDBD,LCDCCD,LCDBL
        ret

;-----------------------------------------------------------------------------
; dis_Th        Display word in hexadecimal
;
;            T  number to display (R18,R19) R18:H, R19:L
;            Y  position on display
;
;          R16 modified
;-----------------------------------------------------------------------------
dis_Th: mov     r16,r18     ;High byte (15..8)
        rcall   dis_R16h
        mov     r16,r19     ;Low byte (7..0)
;-----------------------------------------------------------------------------
; dis_R16h      Display byte in hexadecimal
;
;          R16  number to display
;            Y  position on display
;
;               no register modified
;-----------------------------------------------------------------------------
dis_R16h:
        rcall   dislsb
dismsb: swap    r16
dislsb: push    r16         ;Display nibble
        andi    r16,0x0f
dis4bits:
		LDI_Z   tabseg
		add		r30,r16
		lpm     r16,z

        ror     r16     ;A
        rol     r15
        ror     r16     ;B
        rol     r14
        ror     r16     ;C
        rol     r13
        clc             ;DP
        rol     r12
        ror     r16     ;D
        rol     r12
        ror     r16     ;E
        rol     r13
        ror     r16     ;F
        rol     r15
        ror     r16     ;G
        rol     r14

        pop     r16
        ret
;-----------------------------------------------------------------------------
; kbrd          Wait for a key (debounced)
;
;           R16 key code
;-----------------------------------------------------------------------------
kbrd:   ;wait for previous key to be released
        push    r9
        push    r10
        push    r11

k_release:
        rcall   getscan

        tst     r9
        brne    k_release
        tst     r10
        brne    k_release
        tst     r11
        brne    k_release

k_pressed:
        rcall   getscan

        cp      r10,r11         ;t-1 = t-2 ?
        brne    k_pressed
        cp      r9,r10          ;t = t-1 ?
        brne    k_pressed

        tst     r10
        breq    k_pressed

k_ready:
;        ldi     r16,0x1F
;        and     r11,r16      ;1F

;        LDI_Z   tabkey
;   		add		r30,r11
;		lpm     r16,z
        mov     r16,r11

        pop     r11
        pop     r10
        pop     r9
        ret
;-----------------------------------------------------------------------------
getscan:
        cli
        mov     r9,r4
        mov     r10,r5
        mov     r11,r6
        sei
        ret
;-----------------------------------------------------------------------------
tabseg: .byte 0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x7f,0x6f
tabasc: .byte 0x77,0x7c,0x39,0x5e,0x79,0x71   ;A-F
        .byte 0x3d,0x76,0x06,0x1e,0x72        ;G-K
        .byte 0x38,0x37,0x54,0x3f,0x73        ;L-P
        .byte 0x67,0x50,0x6d,0x78,0x3e        ;Q-U
        .byte 0x1c,0x9c,0xf6,0x6e,0xdb        ;V-Z

;encode the columns into 2-bit code
tabkey: .byte 0xFF,0xFF,0xFF,0xFF
        .byte 0xFF,0xFF,0xFF,0xFF
        .byte 0xFF,0xFF,0xFF,0xFF
        .byte 0xFF,0xFF,0xFF,0x04
        .byte 0xFF,0xFF,0xFF,0xFF
        .byte 0xFF,0xFF,0xFF,0x03
        .byte 0xFF,0xFF,0xFF,0x02
        .byte 0xFF,0x01,0x00,0xFF

        .balign 2   ;always put this at end of tables located in code (flash)

